/*******************************************************************************
*
*
* File:        hal_i2c_slave.h
* Description  i2c hal file
* Version      V0.1
* Date         2021-10-14
* Author       zhanghz
*******************************************************************************/

#ifndef __HAL_I2C_SLAVE_H__
#define __HAL_I2C_SLAVE_H__

#include "tau_device_datatype.h"
#include "tau_common.h"
#include "string.h"

typedef enum
{
    I2C_S_INT_READ = 0,             //发生 读请求 中断
    I2C_S_INT_RX,                   //发生 接收 中断
    I2C_S_INT_STOP                  //发生 stop 中断
} e_i2c_s_int_status;

#if defined(ISP_568) || defined(ISP_368)
typedef enum
{
    I2C_S_0 = 0,
    I2C_S_1,
    I2C_S_MAX
} i2c_s_index_e;
#endif

typedef void (*hal_i2c_s_callback_t)(e_i2c_s_int_status int_status, size_t receive_num);

/**************************************************************************
* @name : hal_i2c_s_init
* @brief : i2c slave 初始化
* @param[in] : slave_addr：从机地址
* @param[in] : addr_bits：从机地址位数
* @return :
* @retval :
**************************************************************************/
void hal_i2c_s_init(uint8_t slave_addr, uint8_t addr_bits);

/**************************************************************************
* @name : hal_i2c_s_dma_write
* @brief : i2c slave dma 发送数据
* @param[in] : txBuffer：发送数据buffer
* @param[in] : data_size：发送数据个数
* @return : STATUS_SUCCESS：数据已排入 DMA 通道，但不一定全部发送
* @return : 其它：发送出错，需要重新调用函数发送
* @retval :
**************************************************************************/
status_t hal_i2c_s_dma_write(const uint8_t *txBuffer, size_t data_size);

/**************************************************************************
* @name : hal_i2c_s_nonblocking_read
* @brief : i2c slave 准备接收数据
* @param[in] : rxBuffer：接收数据buffer
* @param[in] : data_size：接收数据最大个数
* @return : STATUS_SUCCESS：已配置准备接收，此时通信不一定开始
* @return : 其它：接收配置出错，需要重新调用函数配置
* @retval :
**************************************************************************/
status_t hal_i2c_s_nonblocking_read(uint8_t *rxBuffer, size_t data_size);

/**************************************************************************
* @name : hal_i2c_s_transfer_complate
* @brief : 获取 i2c slave 发送状态
* @param[in] :
* @return : true：数据发送完成
* @return : false：数据还在发送
* @retval :
**************************************************************************/
bool hal_i2c_s_write_complate(void);

/**************************************************************************
* @name : hal_i2c_s_read_complate
* @brief : 获取 i2c slave 接收状态
* @param[in] :
* @return : 数据接收个数
* @retval :
**************************************************************************/
uint8_t hal_i2c_s_read_complate(void);

/**************************************************************************
* @name : hal_i2c_s_read_complate_clear
* @brief : 清除 i2c slave 接收状态
* @param[in] :
* @return :
* @retval :
**************************************************************************/
void hal_i2c_s_read_complate_clear(void);

/**************************************************************************
* @name : hal_i2c_s_set_dma_tx_cycle
* @brief : 配置 I2C cycle 模式
* @param[in] :
* @return : ENABLE：cycle模式，DISABLE：非cycle模式
* @retval :
**************************************************************************/
void hal_i2c_s_set_dma_tx_cycle(bool enable);

/**************************************************************************
* @name : hal_i2c_s_set_transfer
* @brief : 配置 i2c 从机数据解析函数
* @param[in] :hal_tp_transfer_phone_tmp：解析函数指针
* @return :
* @retval :
**************************************************************************/
void hal_i2c_s_set_transfer(hal_i2c_s_callback_t hal_i2c_s_callback_tmp);

/**************************************************************************
* @name : hal_i2c_s_read_data
* @brief :read data
* @param[in] : rx_data: 接收数据
* @return : 1: 成功获取数据
* @return : 0: 接收 fifo 为空
* @retval :
**************************************************************************/
status_t hal_i2c_s_read_data(uint8_t *rx_data);

/**************************************************************************
* @name : hal_i2c_s_write_data
* @brief :write data
* @param[in] : tx_data: 准备发送的数据
* @return : 1: 配置发送成功
* @return : 0: 发送 fifo 已满
* @retval :
**************************************************************************/
status_t hal_i2c_s_write_data(const uint8_t tx_data);

/**************************************************************************
 * @name : hal_i2c_s_rxfifo_notempty
 * @brief : 判断当前 rxfifo 中是否有数据
 * @param[in] :
 * @return : true: rxfifo 中有数据
 * @return : false: rxfifo 中没有数据
 * @retval :
 **************************************************************************/
bool hal_i2c_s_rxfifo_notempty(void);

/**************************************************************************
* @name : hal_i2c_s_set_high_impedance
* @brief : 将 I2C 从机的IO口设置为高阻态
* @param[in] :
* @return :
* @retval :
**************************************************************************/
void hal_i2c_s_set_high_impedance(void);
/**************************************************************************
* @name : hal_i2c_s_get_tx_byte_num
* @brief : 获取I2C从机发送成功字节数
* @param[in] :
* @return :发送总字节数
* @retval :
**************************************************************************/
int hal_i2c_s_get_tx_byte_num(void);
/**************************************************************************
* @name : hal_i2c_s_deinit
* @brief : i2c IP去初始化（关掉使能、外设时钟）
* @param[in] :slave_num 从机序号
* @return :
* @retval :
***************************************************************************/
void hal_i2c_s_deinit(void);
#if defined(ISP_568) || defined(ISP_368)
    /**************************************************************************
    * @name : hal_i2c_s_sel
    * @brief : i2c slave 选择
    * @param[in] : slaver：从机编号
    * @return :
    * @retval :
    **************************************************************************/
    void hal_i2c_s_sel(i2c_s_index_e slaver);
#endif
#endif /* __HAL_I2C_SLAVE_H__*/

